@@ -81,7 +81,7 @@ BaseColorMapPlotWnd::setupToolBar()
   // This tool bar button will trigger the transpose axes operation on all the
   // plot widget that are pinned-down.
 
-  const QIcon transposeAxesIcon = QIcon(":/images/transpose-xy-axes.png");
+  const QIcon transposeAxesIcon = QIcon(":/images/svg/transpose-xy-axes.svg");
   mp_transposeAxesAct           = new QAction(
     transposeAxesIcon, tr("Transpose axes for pinned-down plot widgets"), this);
   mp_transposeAxesAct->setStatusTip(
@@ -265,7 +265,7 @@ BaseColorMapPlotWnd::integrateToRt(QCPAbstractPlottable *parent_plottable_p,
 
 
 void
-BaseColorMapPlotWnd::integrateToRtMz(QCPAbstractPlottable *parent_plottable_p,
+BaseColorMapPlotWnd::integrateToMzRt(QCPAbstractPlottable *parent_plottable_p,
                                      const ProcessingFlow &processing_flow)
 {
   // qDebug();
@@ -282,13 +282,13 @@ BaseColorMapPlotWnd::integrateToRtMz(QCPAbstractPlottable *parent_plottable_p,
   //"color map plot with processing flow:"
   //<< processing_flow.toString();
 
-  mp_programWindow->integrateToRtMz(
+  mp_programWindow->integrateToMzRt(
     parent_plottable_p, nullptr, processing_flow);
 }
 
 
 void
-BaseColorMapPlotWnd::integrateToRtDt(QCPAbstractPlottable *parent_plottable_p,
+BaseColorMapPlotWnd::integrateToDtRt(QCPAbstractPlottable *parent_plottable_p,
                                      const ProcessingFlow &processing_flow)
 {
   // qDebug();
@@ -300,7 +300,7 @@ BaseColorMapPlotWnd::integrateToRtDt(QCPAbstractPlottable *parent_plottable_p,
   // handles their own data. So we just relay this call to the main program
   // window that will in turn relay it to the mass spec plot window.
 
-  mp_programWindow->integrateToRtDt(
+  mp_programWindow->integrateToDtRt(
     parent_plottable_p, nullptr, processing_flow);
 }
 
