@@ -172,7 +172,7 @@ BasePlotCompositeWidget::createMainMenu()
 
   // Create the menu push button.
   const QIcon main_menu_icon =
-    QIcon(":/images/mobile-phone-like-menu-button.svg");
+    QIcon(":/images/svg/mobile-phone-like-menu-button.svg");
   m_ui.mainMenuPushButton->setIcon(main_menu_icon);
   m_ui.mainMenuPushButton->setToolTip("Plot widget menu (Ctrl+P, M)");
   m_ui.mainMenuPushButton->setShortcut(QKeySequence("Ctrl+P, M"));
@@ -462,23 +462,61 @@ BasePlotCompositeWidget::xAxisMeasurement(
 {
   // A measurement was performed, inform the user.
 
+  // There are several situations:
+  //
+  // 1. The selection polygon is 1D, we only care of the X axis segment
+  // coordinates and delta.
+  //
+  // 2. The selection polygon is 2D, we should care of both the X axis segment
+  // coordinates and of the Y axis segment coordinates, also show both deltas.
+
   // Note that we want to provide the [x -- x] x axis range in sorted order,
   // but the data in the context are not sorted. So craft a QCPRange object
   // that will automagically sort the values. Then use it to craft the
   // message.
 
-  QCPRange sorted_range(context.xRegionRangeStart, context.xRegionRangeEnd);
+#if 0
+
+  // This was the previous version that only cared of X axis movements, but
+  // with the selection polygon we now want to show the full extent of the data:
+  // X axis and Y axis. 
+  QCPRange sorted_range(context.m_xRegionRangeStart, context.m_xRegionRangeEnd);
 
   QString message = QString("[%1--%2]")
                       .arg(sorted_range.lower, 0, 'f', 3)
                       .arg(sorted_range.upper, 0, 'f', 3);
 
   if(with_delta)
-    message.append(QString(" - delta: %1").arg(context.xDelta, 0, 'f', 3));
+    message.append(QString(" - delta: %1").arg(context.m_xDelta, 0, 'f', 3));
 
   // qDebug() << message;
 
   m_ui.statusLineEdit->setText(message);
+#endif
+
+
+  // First describe the selection polygon.
+
+  QString text = context.m_selectionPolygon.toShort4PointsString();
+
+  // Now describe the x|y delta.
+
+  double start;
+  double end;
+
+  // We always want the X delta.
+  context.m_selectionPolygon.rangeX(start, end);
+  text += QString(" -- xDelta: %1").arg(end - start, 0, 'f', 3);
+
+  // We want the Y delta only if the polygon is 2D.
+  if(context.m_selectionPolygon.is2D() && with_delta)
+    {
+      //qDebug() << "Selection polygon is 2D";
+      context.m_selectionPolygon.rangeY(start, end);
+      text += QString(" -- yDelta: %1").arg(end - start, 0, 'f', 3);
+    }
+
+  m_ui.statusLineEdit->setText(text);
 }
 
 
@@ -488,9 +526,9 @@ BasePlotCompositeWidget::plotWidgetKeyPressEvent(
 {
   // qDebug() << "The context" << context.toString();
 
-  m_plotWidgetPressedKeyCode = context.pressedKeyCode;
+  m_plotWidgetPressedKeyCode = context.m_pressedKeyCode;
 
-  if(context.pressedKeyCode == Qt::Key_Space)
+  if(context.m_pressedKeyCode == Qt::Key_Space)
     {
       // qDebug();
       craftAnalysisStanza(context);
@@ -502,7 +540,7 @@ void
 BasePlotCompositeWidget::plotWidgetKeyReleaseEvent(
   const pappso::BasePlotContext &context)
 {
-  m_plotWidgetReleasedKeyCode = context.releasedKeyCode;
+  m_plotWidgetReleasedKeyCode = context.m_releasedKeyCode;
   // qDebug() << "Released key:" << m_plotWidgetReleasedKeyCode;
 }
 
@@ -513,13 +551,13 @@ BasePlotCompositeWidget::plotWidgetMouseReleaseEvent(
 {
   // qDebug();
 
-  // if(context.lastPressedMouseButtonsUponRelease == Qt::NoButton)
+  // if(context.m_lastPressedMouseButtonsUponRelease == Qt::NoButton)
   // qDebug() << "no button";
 
-  // if(context.lastPressedMouseButtonsUponRelease == Qt::LeftButton)
+  // if(context.m_lastPressedMouseButtonsUponRelease == Qt::LeftButton)
   // qDebug() << "left button";
 
-  // if(context.lastPressedMouseButtonsUponRelease == Qt::RightButton)
+  // if(context.m_lastPressedMouseButtonsUponRelease == Qt::RightButton)
   //{
   // qDebug() << "right button with context:" << context.toString();
   //}
@@ -527,7 +565,7 @@ BasePlotCompositeWidget::plotWidgetMouseReleaseEvent(
   // Now check what keyboard key was being pressed while the mouse
   // operation was going on.
 
-  // int pressed_key_code = context.pressedKeyCode;
+  // int pressed_key_code = context.m_pressedKeyCode;
   // qDebug() << "pressed_key_code:" << pressed_key_code;
 }
 
@@ -576,25 +614,20 @@ BasePlotCompositeWidget::getProcessingFlowForPlottable(
   // Each graph has its own ProcessingFlow, and the association is in the
   // plottableProcessingFlowMap.
 
-  ProcessingFlow processing_flow;
+  std::map<QCPAbstractPlottable *, ProcessingFlow>::const_iterator it;
 
-  auto result = std::find_if(
-    m_plottableProcessingFlowMap.begin(),
-    m_plottableProcessingFlowMap.end(),
-    [plottable_p](
-      const std::pair<QCPAbstractPlottable *, ProcessingFlow> &item) {
-      return item.first == plottable_p;
-    });
-
-  if(result != m_plottableProcessingFlowMap.end())
+  for(it = m_plottableProcessingFlowMap.begin();
+      it != m_plottableProcessingFlowMap.end();
+      ++it)
     {
-      // qDebug().noquote() << "Found the processing flow for graph:"
-      //<< result->second.toString();
-
-      return result->second;
+      if(it->first == plottable_p)
+        {
+          return it->second;
+          break;
+        }
     }
-  else
-    qFatal("Cannot be that graph has no processing flow associated to it.");
+
+  qFatal("Cannot be that graph has no processing flow associated to it.");
 
   return ProcessingFlow();
 }
@@ -763,18 +796,19 @@ BasePlotCompositeWidget::showMsFragmentationSpecDlg()
 
   MsFragmentationSpec ms_fragmentation_spec;
 
-  // Ask that only specs having a valid MsFragmentationSpec instance be
-  // returned (true boolean below)
+  // Ask that only steps having a valid MsFragmentationSpec instance be
+  // returned (true boolean below). Only of interest are step with the
+  // destination matching MZ.
+
+  std::vector<const ProcessingStep *> processing_steps =
+    processing_flow.stepsMatchingDestType(ProcessingType("MZ"), true);
 
-  std::vector<ProcessingSpec *> processing_specs =
-    processing_step_p->allProcessingSpecsMatching(ProcessingType("ANY_TO_MZ"),
-                                                  true);
   // qDebug() << "The number of processing specs that match processing
   // type "
   //"\"ANY_TO_MZ\" is:"
   //<< processing_specs.size();
 
-  if(!processing_specs.size())
+  if(!processing_steps.size())
     {
       // Not a single spec had a valid ms fragmentation spec. Last resort:
       // get the default one from the flow.
@@ -789,11 +823,11 @@ BasePlotCompositeWidget::showMsFragmentationSpecDlg()
     }
   else
     {
-      // There was/were indeed spec instances with a valid ms
+      // There was/were indeed step instances with a valid ms
       // fragmentation spec. Arbitrarily select the last one.
 
-      ProcessingSpec *processing_spec_p = processing_specs.back();
-      ms_fragmentation_spec = processing_spec_p->getMsFragmentationSpec();
+      processing_step_p     = processing_steps.back();
+      ms_fragmentation_spec = processing_step_p->getMsFragmentationSpec();
     }
 
   // We can finally use it to  configure the dialog window.
@@ -860,7 +894,7 @@ BasePlotCompositeWidget::msFragmentationSpecChanged(
   // Set the default ms frag spec.
   processing_flow_p->setDefaultMsFragmentationSpec(ms_fragmentation_spec);
 
-  qDebug() << "Now setting the MS level to the spin box.";
+  //qDebug() << "Now setting the MS level to the spin box.";
 
   // Change the MS level value in the spinbox.
   // But we need to disconnect the spin box' value changed signal, because
@@ -868,9 +902,9 @@ BasePlotCompositeWidget::msFragmentationSpecChanged(
   // frag spec's ms level again.
 
   disconnect(m_ui.msLevelSpinBox,
-          QOverload<int>::of(&QSpinBox::valueChanged),
-          this,
-          &BasePlotCompositeWidget::msLevelValueChanged);
+             QOverload<int>::of(&QSpinBox::valueChanged),
+             this,
+             &BasePlotCompositeWidget::msLevelValueChanged);
 
   m_ui.msLevelSpinBox->setValue(ms_fragmentation_spec.getMsLevel());
 
@@ -879,8 +913,6 @@ BasePlotCompositeWidget::msFragmentationSpecChanged(
           QOverload<int>::of(&QSpinBox::valueChanged),
           this,
           &BasePlotCompositeWidget::msLevelValueChanged);
-
-
 }
 
 
@@ -890,7 +922,7 @@ BasePlotCompositeWidget::msLevelValueChanged(int value)
   // The user has selected a new MS level value, we need to ensure that this
   // value is record in the MsFragmentationSpec.
 
-  qDebug() << "Value: " << value;
+  //qDebug() << "Value: " << value;
 
   QCPAbstractPlottable *source_plottable_p =
     plottableToBeUsedAsIntegrationSource();
@@ -1161,7 +1193,7 @@ BasePlotCompositeWidget::craftAnalysisStanza(
   // We need to know what kind of data we are handling so that we can select the
   // proper format.
 
-  pappso::DataKind data_kind = context.dataKind;
+  pappso::DataKind data_kind = context.m_dataKind;
 
   // qDebug() << "data_kind:" << static_cast<int>(data_kind);
 
@@ -1281,13 +1313,13 @@ BasePlotCompositeWidget::craftAnalysisStanza(
           // plot widget. Let that handling to the derived class.
           if(curChar == 'x')
             {
-              stanza += QString("%1").arg(context.xDelta, 0, 'g', 6);
+              stanza += QString("%1").arg(context.m_xDelta, 0, 'g', 6);
               prevChar = ' ';
               continue;
             }
           if(curChar == 'y')
             {
-              stanza += QString("%1").arg(context.yDelta, 0, 'g', 6);
+              stanza += QString("%1").arg(context.m_yDelta, 0, 'g', 6);
 
               prevChar = ' ';
               continue;
@@ -1302,14 +1334,15 @@ BasePlotCompositeWidget::craftAnalysisStanza(
             }
           if(curChar == 'b')
             {
-              stanza += QString("%1").arg(context.xRegionRangeStart, 0, 'f', 3);
+              stanza +=
+                QString("%1").arg(context.m_xRegionRangeStart, 0, 'f', 3);
 
               prevChar = ' ';
               continue;
             }
           if(curChar == 'e')
             {
-              stanza += QString("%1").arg(context.xRegionRangeEnd, 0, 'f', 3);
+              stanza += QString("%1").arg(context.m_xRegionRangeEnd, 0, 'f', 3);
 
               prevChar = ' ';
               continue;
