@@ -143,6 +143,16 @@ BasePlotWnd::initialize()
 {
   setWindowTitle(QString("mineXpert2 - %1").arg(m_title));
 
+  // This attribute make sure that the main window of the program is destroyed
+  // when it is closed. Effectively stopping the program.
+  setAttribute(Qt::WA_DeleteOnClose);
+
+  // The default window icon.
+  QString pixmap_file_name = ":/images/icons/32x32/minexpert2.png";
+  QPixmap icon_pixmap(pixmap_file_name);
+  QIcon icon(icon_pixmap);
+  setWindowIcon(icon);
+
   // This will take care of the main menu that sits in the first button of the
   // tool bar.
   setupToolBar();
@@ -218,8 +228,9 @@ BasePlotWnd::setupToolBar()
 
   // The tool bar non-menu buttons
 
-  const QIcon xRangeIcon = QIcon(":/images/lock-x-range.png");
-  mp_lockXRangeAct       = new QAction(xRangeIcon, tr("Lock X range (Ctrl+L, X)"), this);
+  const QIcon xRangeIcon = QIcon(":/images/svg/lock-x-range.svg");
+  mp_lockXRangeAct =
+    new QAction(xRangeIcon, tr("Lock X range (Ctrl+L, X)"), this);
   mp_lockXRangeAct->setCheckable(true);
   mp_lockXRangeAct->setShortcut(QKeySequence("Ctrl+L, X"));
   mp_lockXRangeAct->setStatusTip(tr("Lock X range"));
@@ -229,8 +240,9 @@ BasePlotWnd::setupToolBar()
 
   mp_toolBar->addAction(mp_lockXRangeAct);
 
-  const QIcon yRangeIcon = QIcon(":/images/lock-y-range.png");
-  mp_lockYRangeAct       = new QAction(yRangeIcon, tr("Lock Y range (Ctrl+L, Y)"), this);
+  const QIcon yRangeIcon = QIcon(":/images/svg/lock-y-range.svg");
+  mp_lockYRangeAct =
+    new QAction(yRangeIcon, tr("Lock Y range (Ctrl+L, Y)"), this);
   mp_lockYRangeAct->setCheckable(true);
   mp_lockYRangeAct->setShortcut(QKeySequence("Ctrl+L, Y"));
   mp_lockYRangeAct->setStatusTip(tr("Lock Y range"));
@@ -253,7 +265,7 @@ BasePlotWnd::createMainMenu()
   mp_mainMenu = new QMenu(this);
 
   const QIcon main_menu_icon =
-    QIcon(":/images/mobile-phone-like-menu-button.svg");
+    QIcon(":/images/svg/mobile-phone-like-menu-button.svg");
   mp_mainMenuPushButton = new QPushButton(main_menu_icon, QString(""), this);
   mp_mainMenuPushButton->setToolTip("Plot widget menu (Ctrl+W, M)");
   mp_mainMenuPushButton->setShortcut(QKeySequence("Ctrl+W, M"));
@@ -270,8 +282,8 @@ BasePlotWnd::createMainMenu()
   // We want to be able to set the mz integ params for all the plot widgets in
   // the window in one go.
 
-  QAction *new_act_p =
-    new QAction("Set m/z &integration params for all pinned-down widgets", this);
+  QAction *new_act_p = new QAction(
+    "Set m/z &integration params for all pinned-down widgets", this);
   new_act_p->setShortcut(QKeySequence("Ctrl+I, P"));
 
   connect(new_act_p, &QAction::triggered, [this]() {
@@ -379,8 +391,10 @@ BasePlotWnd::createMainMenu()
   QMenu *remove_plot_widgets_menu_p = new QMenu("Remove plot &widgets...");
   mp_plotWidgetManagementMenu->addMenu(remove_plot_widgets_menu_p);
 
-  QAction *remove_pinned_plot_widgets_p = new QAction(tr("&Remove pinned-down plot widgets"), this);
-  remove_pinned_plot_widgets_p->setStatusTip(tr("Remove pinned-down plot widgets"));
+  QAction *remove_pinned_plot_widgets_p =
+    new QAction(tr("&Remove pinned-down plot widgets"), this);
+  remove_pinned_plot_widgets_p->setStatusTip(
+    tr("Remove pinned-down plot widgets"));
   remove_pinned_plot_widgets_p->setShortcut(QKeySequence("Ctrl+R, P"));
 
   connect(remove_pinned_plot_widgets_p, &QAction::triggered, [this]() {
@@ -396,16 +410,19 @@ BasePlotWnd::createMainMenu()
     new QAction(tr("Remove pinned-down plot widgets and &descendants"), this);
   remove_pinned_plot_widgets_descendants_p->setStatusTip(
     tr("Remove pinned-down plot widgets and descendants"));
-  remove_pinned_plot_widgets_descendants_p->setShortcut(QKeySequence("Ctrl+R, Ctrl+P, D"));
+  remove_pinned_plot_widgets_descendants_p->setShortcut(
+    QKeySequence("Ctrl+R, Ctrl+P, D"));
 
-  connect(remove_pinned_plot_widgets_descendants_p, &QAction::triggered, [this]() {
-    std::vector<BasePlotCompositeWidget *> widget_list = pinnedDownWidgets();
+  connect(
+    remove_pinned_plot_widgets_descendants_p, &QAction::triggered, [this]() {
+      std::vector<BasePlotCompositeWidget *> widget_list = pinnedDownWidgets();
 
-    for(auto &&widget_p : widget_list)
-      plotCompositeWidgetDestructionRequested(widget_p);
-  });
+      for(auto &&widget_p : widget_list)
+        plotCompositeWidgetDestructionRequested(widget_p);
+    });
 
-  remove_plot_widgets_menu_p->addAction(remove_pinned_plot_widgets_descendants_p);
+  remove_plot_widgets_menu_p->addAction(
+    remove_pinned_plot_widgets_descendants_p);
 
   remove_plot_widgets_menu_p->addSeparator();
 
@@ -422,7 +439,8 @@ BasePlotWnd::createMainMenu()
   remove_plot_widgets_menu_p->addAction(mp_removeEmptyPlotWidgetsAct);
 
 
-  mp_removeAllPlotWidgetsAct = new QAction(tr("Remove &all plot widgets"), this);
+  mp_removeAllPlotWidgetsAct =
+    new QAction(tr("Remove &all plot widgets"), this);
   mp_removeAllPlotWidgetsAct->setStatusTip(tr("Remove all plot widgets"));
   mp_removeAllPlotWidgetsAct->setShortcut(QKeySequence("Ctrl+R, A"));
 
@@ -440,7 +458,8 @@ BasePlotWnd::createMainMenu()
     new QAction(tr("Remove all plot widgets and d&escendants"), this);
   mp_removeAllPlotWidgetsAndDescendantsAct->setStatusTip(
     tr("Remove all plot widgets and descendants"));
-  mp_removeAllPlotWidgetsAndDescendantsAct->setShortcut(QKeySequence("Ctrl+R, Ctrl+A, D"));
+  mp_removeAllPlotWidgetsAndDescendantsAct->setShortcut(
+    QKeySequence("Ctrl+R, Ctrl+A, D"));
 
   connect(
     mp_removeAllPlotWidgetsAndDescendantsAct, &QAction::triggered, [this]() {
@@ -872,7 +891,8 @@ BasePlotWnd::integrateToTicIntensity(
       new QualifiedMassSpectrumVectorMassDataIntegratorToTicInt(
         ms_run_data_set_csp, processing_flow, qualified_mass_spectra_sp);
 
-  mass_data_integrator_p->setMaxThreadUseCount(mp_programWindow->getMaxThreadUseCount());
+  mass_data_integrator_p->setMaxThreadUseCount(
+    mp_programWindow->getMaxThreadUseCount());
 
   // Ensure the mass data integrator messages are used.
 
@@ -1209,16 +1229,16 @@ BasePlotWnd::plotRangesChanged(const pappso::BasePlotContext &context)
   int which_plot_axis = 0;
 
   if(m_isLockedXRange)
-    which_plot_axis |= static_cast<int>(pappso::PlotAxis::x_axis);
+    which_plot_axis |= static_cast<int>(pappso::Axis::x);
 
   if(m_isLockedYRange)
-    which_plot_axis |= static_cast<int>(pappso::PlotAxis::y_axis);
+    which_plot_axis |= static_cast<int>(pappso::Axis::y);
 
   if(!which_plot_axis)
     return;
 
-  QCPRange x_axis_range(context.xRange.lower, context.xRange.upper);
-  QCPRange y_axis_range(context.yRange.lower, context.yRange.upper);
+  QCPRange x_axis_range(context.m_xRange.lower, context.m_xRange.upper);
+  QCPRange y_axis_range(context.m_yRange.lower, context.m_yRange.upper);
 
   QList<BasePlotCompositeWidget *> widget_list =
     findChildren<BasePlotCompositeWidget *>("plotCompositeWidget");
@@ -1237,9 +1257,7 @@ BasePlotWnd::plotRangesChanged(const pappso::BasePlotContext &context)
         }
 
       widget_p->mp_plotWidget->replotWithAxesRanges(
-        x_axis_range,
-        y_axis_range,
-        static_cast<pappso::PlotAxis>(which_plot_axis));
+        x_axis_range, y_axis_range, static_cast<pappso::Axis>(which_plot_axis));
     }
 }
 
@@ -1293,49 +1311,74 @@ BasePlotWnd::fillInQualifiedMassSpectraVector(
   // If the vector is not empty, then we return its size without modification.
 
   if(qualified_mass_spectra_sp->size())
-    return qualified_mass_spectra_sp->size();
+    {
+      qDebug() << "The vector of qualified spectra is not empty. Returning "
+                  "with no modification.";
 
-  // Now, we know we can fill in the vector.
+      return qualified_mass_spectra_sp->size();
+    }
 
-  // We get a processing flow from which we extract the innermost rt and dt
-  // ranges. We use those ranges for selecting the most restricted set of
-  // qualified mass spectra so as to speed up the integrations.
+  // Now, we know we can fill in the vector. But try to limit as much as
+  // possible the number of spectra to add. We do that by only pushing into the
+  // vector the spectra that have rt (and dt, if mobility data) values matching
+  // the innermost rt (and dt) ranges.
 
-  double start_rt = std::numeric_limits<double>::infinity();
-  double end_rt   = std::numeric_limits<double>::infinity();
+  std::pair<double, double> range_pair;
 
-  bool integration_rt = processing_flow.innermostRtRange(start_rt, end_rt);
+  // We choose RT_TO_ANY that matches in the widest possible manner all the
+  // previous integrations that started with a rt-involving selection (both
+  // mono- and two-dimensional, either from the ms run table view or any
+  // trace/color map plot).
 
-  if(!integration_rt)
+  bool integration =
+    processing_flow.innermostRange("ANY_RT", range_pair);
+
+  if(!integration)
     qFatal("Programming error.");
 
-  // std::size_t added_rt_mass_spectra =
-  ms_run_data_set_csp->getMsRunDataSetTreeCstSPtr()
-    ->addDataSetQualMassSpectraInsideDtRtRange(
-      start_rt, end_rt, *qualified_mass_spectra_sp, pappso::DataKind::rt);
+  qDebug() << "Filling-in mass spectra for RT range: " << range_pair.first
+           << "-" << range_pair.second;
 
-  // qDebug() << "Added RT mass spectra:" << added_rt_mass_spectra;
+  std::size_t added_rt_mass_spectra =
+    ms_run_data_set_csp->getMsRunDataSetTreeCstSPtr()
+      ->addDataSetQualMassSpectraInsideDtRtRange(range_pair.first,
+                                                 range_pair.second,
+                                                 *qualified_mass_spectra_sp,
+                                                 pappso::DataKind::rt);
+
+  qDebug() << "Added RT mass spectra:" << added_rt_mass_spectra;
 
   // Now see if we can remove some mass spectra on the basis of DT values.
 
-  double start_dt = std::numeric_limits<double>::infinity();
-  double end_dt   = std::numeric_limits<double>::infinity();
+  // We choose ANY_DT that matches in the widest possible manner all the
+  // previous integrations that started with a dt-involving selection (both
+  // mono- and two-dimensionale).
+  integration =
+    processing_flow.innermostRange("ANY_DT", range_pair);
 
-  bool integration_dt = processing_flow.innermostDtRange(start_dt, end_dt);
-
-  if(integration_dt)
+  // Here, it is not an error not to have any DT integration because not all
+  // the mass spectral acquisitions involved ion mobility.
+  if(integration)
     {
-      // qDebug() << "Detected DT integration";
+      qDebug()
+        << "Detected DT integration, removing mass spectra outside DT range: "
+        << range_pair.first << "-" << range_pair.second;
 
-      // std::size_t removed_mass_spectra =
-      ms_run_data_set_csp->getMsRunDataSetTreeCstSPtr()
-        ->removeDataSetQualMassSpectraOutsideDtRtRange(
-          start_dt, end_dt, *qualified_mass_spectra_sp, pappso::DataKind::dt);
+      std::size_t removed_mass_spectra =
+        ms_run_data_set_csp->getMsRunDataSetTreeCstSPtr()
+          ->removeDataSetQualMassSpectraOutsideDtRtRange(
+            range_pair.first,
+            range_pair.second,
+            *qualified_mass_spectra_sp,
+            pappso::DataKind::dt);
 
-      // qDebug() << "Removed mass spectra on basis of DT:"
-      //<< removed_mass_spectra;
+      qDebug() << "Number of mass spectra removed on the basis of DT:"
+               << removed_mass_spectra;
     }
 
+  // At this point we need to really check which of the remaining spectra are
+  // inside of the selection polygon.
+
   return qualified_mass_spectra_sp->size();
 }
 
