@@ -93,10 +93,10 @@ BaseTracePlotCompositeWidget::BaseTracePlotCompositeWidget(
   // tricky to handle because it is not possible to know if it is checed or not.
 
   const QIcon plus_combine_icon =
-    QIcon(":/images/keep-trace-and-plus-combine-new-one-32x32.png");
+    QIcon(":/images/svg/keep-trace-and-plus-combine-new-one.svg");
 
   const QIcon minus_combine_icon =
-    QIcon(":/images/keep-trace-and-minus-combine-new-one-32x32.png");
+    QIcon(":/images/svg/keep-trace-and-minus-combine-new-one.svg");
 
   m_ui.combinationsComboBox->insertItem(
     -1, minus_combine_icon, "", static_cast<int>(TraceCombinationSign::MINUS));
@@ -487,11 +487,11 @@ BaseTracePlotCompositeWidget::plotWidgetKeyPressEvent(
 
   BasePlotCompositeWidget::plotWidgetKeyPressEvent(context);
 
-  m_plotWidgetPressedKeyCode = context.pressedKeyCode;
+  m_plotWidgetPressedKeyCode = context.m_pressedKeyCode;
 
   // qDebug() << "The context" << context.toString();
 
-  if(context.pressedKeyCode == Qt::Key_Space)
+  if(context.m_pressedKeyCode == Qt::Key_Space)
     {
       // At the moment, in this class we have nothing very specific to do. But
       // we could reparse the m_analysisStanza string and fill-in any remaining
@@ -531,8 +531,8 @@ BaseTracePlotCompositeWidget::plotWidgetKeyReleaseEvent(
       if(mp_plotWidget)
         {
           QPointF mouseCursorPos(
-            mp_plotWidget->xAxis->coordToPixel(context.currentDragPoint.x()),
-            mp_plotWidget->xAxis->coordToPixel(context.currentDragPoint.y()));
+            mp_plotWidget->xAxis->coordToPixel(context.m_currentDragPoint.x()),
+            mp_plotWidget->xAxis->coordToPixel(context.m_currentDragPoint.y()));
 
           QMouseEvent *mouse_event_p = new QMouseEvent(QEvent::MouseButtonPress,
                                                        mouseCursorPos,
