@@ -52,6 +52,7 @@
 #include <pappsomspp/processing/combiners/traceminuscombiner.h>
 #include <pappsomspp/processing/combiners/massspectrumpluscombiner.h>
 #include <pappsomspp/processing/combiners/massspectrumminuscombiner.h>
+#include <pappsomspp/widget/plotwidget/massspectraceplotcontext.h>
 
 
 /////////////////////// Local includes
@@ -204,7 +205,8 @@ MassSpecTracePlotCompositeWidget::createMainMenu()
     "Set charge &minimal fractional part", dynamic_cast<QObject *>(this));
   set_charge_minimal_fractional_part_action_p->setStatusTip(
     "Set the charge minimal fractional part (typically 0.99)");
-  set_charge_minimal_fractional_part_action_p->setShortcut(QKeySequence("Ctrl+S, M"));
+  set_charge_minimal_fractional_part_action_p->setShortcut(
+    QKeySequence("Ctrl+S, M"));
 
   connect(
     set_charge_minimal_fractional_part_action_p, &QAction::triggered, [this]() {
@@ -656,6 +658,7 @@ MassSpecTracePlotCompositeWidget::gaussianFitOnVisibleMaxima()
     msg, source_plottable_p->pen().color());
 }
 
+
 void
 MassSpecTracePlotCompositeWidget::mainMenuPushButtonClicked()
 {
@@ -686,15 +689,15 @@ MassSpecTracePlotCompositeWidget::plotWidgetKeyPressEvent(
 
   // First call the base class that will fill-in the generic fields if the space
   // char was pressed.
-  BaseTracePlotCompositeWidget::plotWidgetKeyPressEvent(context.baseContext);
+  BaseTracePlotCompositeWidget::plotWidgetKeyPressEvent(context.m_baseContext);
 
   // qDebug() << "After filling-in by base class, the stanza is:"
   //<< m_analysisStanza;
 
-  m_plotWidgetPressedKeyCode = context.baseContext.pressedKeyCode;
+  m_plotWidgetPressedKeyCode = context.m_baseContext.m_pressedKeyCode;
 
   // Now handle here the mass specific data, like z, Mr, m/z.
-  if(context.baseContext.pressedKeyCode == Qt::Key_Space)
+  if(context.m_baseContext.m_pressedKeyCode == Qt::Key_Space)
     {
       QCPAbstractPlottable *plottable_p =
         plottableToBeUsedAsIntegrationSource();
@@ -733,7 +736,7 @@ MassSpecTracePlotCompositeWidget::plotWidgetKeyPressEvent(
       mp_parentWnd->recordAnalysisStanza(stanza, plottable_p->pen().color());
     }
   // End of
-  // if(context.baseContext.pressedKeyCode == Qt::Key_Space)
+  // if(context.m_baseContext.m_pressedKeyCode == Qt::Key_Space)
 }
 
 
@@ -745,10 +748,10 @@ MassSpecTracePlotCompositeWidget::plotWidgetKeyReleaseEvent(
 
   // The release event sets the local key code to 0 so that we know that no
   // keyboard key is being pressed *now*.
-  m_plotWidgetPressedKeyCode = context.pressedKeyCode;
+  m_plotWidgetPressedKeyCode = context.m_pressedKeyCode;
 
   // qDebug() << "The key:" << m_plotWidgetPressedKeyCode
-  //<< "the modifiers:" << context.keyboardModifiers;
+  //<< "the modifiers:" << context.m_keyboardModifiers;
 }
 
 
@@ -764,26 +767,26 @@ MassSpecTracePlotCompositeWidget::plotWidgetMassDeconvolution(
   // We want to append the deconvolution results to the status text.
   // QString new_text =
   // m_ui.statusLineEdit->text() + QString("-- z: %1 - m/z: %2 - Mr: %3")
-  //.arg(context.lastZ)
-  //.arg(context.lastMz)
-  //.arg(context.lastMr);
+  //.arg(context.m_lastZ)
+  //.arg(context.m_lastMz)
+  //.arg(context.m_lastMr);
 
   QString new_text =
     QString("[%1--%2] - delta: %3 -- z: %4 - m/z: %5 - Mr: %6")
-      .arg(std::min<double>(context.baseContext.xRegionRangeStart,
-                            context.baseContext.xRegionRangeEnd),
+      .arg(std::min<double>(context.m_baseContext.m_xRegionRangeStart,
+                            context.m_baseContext.m_xRegionRangeEnd),
            0,
            'f',
            3)
-      .arg(std::max<double>(context.baseContext.xRegionRangeStart,
-                            context.baseContext.xRegionRangeEnd),
+      .arg(std::max<double>(context.m_baseContext.m_xRegionRangeStart,
+                            context.m_baseContext.m_xRegionRangeEnd),
            0,
            'f',
            3)
-      .arg(fabs(context.baseContext.xDelta), 0, 'f', 3)
-      .arg(context.lastZ)
-      .arg(context.lastMz, 0, 'f', 3)
-      .arg(context.lastMr);
+      .arg(fabs(context.m_baseContext.m_xDelta), 0, 'f', 3)
+      .arg(context.m_lastZ)
+      .arg(context.m_lastMz, 0, 'f', 3)
+      .arg(context.m_lastMr);
 
   m_ui.statusLineEdit->setText(new_text);
 }
@@ -795,7 +798,7 @@ MassSpecTracePlotCompositeWidget::plotWidgetResolvingPowerComputation(
 {
   // We want to append the resolving power results to the status text.
   QString new_text = m_ui.statusLineEdit->text() +
-                     QString("-- Rp: %1").arg(context.lastResolvingPower);
+                     QString("-- Rp: %1").arg(context.m_lastResolvingPower);
 
   m_ui.statusLineEdit->setText(new_text);
 }
@@ -824,8 +827,8 @@ MassSpecTracePlotCompositeWidget::moveMouseCursorToNextGraphPoint(
   // in the base class function.
 
   pappso::BasePlotContext local_context = context;
-  local_context.xRegionRangeStart       = m_integrationRange.lower;
-  local_context.xRegionRangeEnd         = m_integrationRange.upper;
+  local_context.m_xRegionRangeStart     = m_integrationRange.lower;
+  local_context.m_xRegionRangeEnd       = m_integrationRange.upper;
 
   if(m_isSinglePointIntegrationMode)
     integrationRequested(local_context);
@@ -836,21 +839,39 @@ void
 MassSpecTracePlotCompositeWidget::integrationRequested(
   const pappso::BasePlotContext &context)
 {
-  // qDebug().noquote() << "context:" << context.toString();
+   qDebug().noquote() << "context:" << context.toString();
 
   // The reference implementation is the TicXicChromTracePlotCompositeWidget
   // one. Look there for all the explanative comments.
 
-  // Immediately check if the selection range makes sense. For example, having
-  // both the start/end values in the negative values does not mean anything
-  // with respect to mass spectrum data!
+  // Make a local copy of the context because we may need to modify it.
+  pappso::BasePlotContext local_context = context;
 
-  // Remember that the xregion start and end are not sorted.
-  double range_start =
-    std::min<double>(context.xRegionRangeStart, context.xRegionRangeEnd);
-  double range_end =
-    std::max<double>(context.xRegionRangeStart, context.xRegionRangeEnd);
+  // Check if the selection polygon contain meaningful data. We are in trace
+  // plot widget, so the selection polygon might be in the form of a
+  // mono-dimensional polygon, although this is not mandatory. But we rely on
+  // the selection polygon to be of 1 or 2 dimension for some optimizations. So,
+  // if the polygon is 2D, convert it to 1D.
 
+  if(local_context.m_selectionPolygon.is2D())
+    qDebug() << "Odd situation where the selection polygon is two-dimensional "
+                "and the trace plot selection is mono-dimensional. Converting "
+                "it to a 1D selection polygon.";
+
+  local_context.m_selectionPolygon.convertTo1D();
+
+  // Sanity check.
+
+  // The user may select at the left of the x=0, but that does not make sense if
+  // the whole selected region is in the x axis negative space.
+  double range_start;
+  double range_end;
+
+  local_context.m_selectionPolygon.rangeX(range_start, range_end);
+
+  qDebug() << "X axis range:" << range_start << "-" << range_end;
+
+  // Having negative retention times does not make any sense.
   if(range_start < 0 && range_end < 0)
     return;
 
@@ -868,7 +889,8 @@ MassSpecTracePlotCompositeWidget::integrationRequested(
       return;
     }
 
-  // Create a local copy of the processing flow.
+  // Create a local copy of the processing flow so that we can modify it by
+  // adding the new processing step that describes what we are doing now.
 
   ProcessingFlow local_processing_flow =
     getProcessingFlowForPlottable(plottable_p);
@@ -881,13 +903,10 @@ MassSpecTracePlotCompositeWidget::integrationRequested(
 
   // qDebug() << ms_run_data_set_csp->getMsRunDataSetStats().toString();
 
-  // Now we need to add a new step in that processing flow so that we document
-  // this new integration that we are performing.
+  // Create a processing step to document the specifics of this new integration.
 
   ProcessingStep *processing_step_p = new ProcessingStep();
 
-  ProcessingSpec *processing_spec_p = new ProcessingSpec();
-
   MsFragmentationSpec ms_fragmentation_spec =
     local_processing_flow.getDefaultMsFragmentationSpec();
 
@@ -898,7 +917,7 @@ MassSpecTracePlotCompositeWidget::integrationRequested(
       // flow " "is valid, using it : "
       //<< ms_fragmentation_spec.toString();
 
-      processing_spec_p->setMsFragmentationSpec(ms_fragmentation_spec);
+      processing_step_p->setMsFragmentationSpec(ms_fragmentation_spec);
     }
   else
     {
@@ -906,15 +925,24 @@ MassSpecTracePlotCompositeWidget::integrationRequested(
       //"valid. Not using it.";
     }
 
+  // Now make sure we document the selection polygon.
   // Now define the graph range for the integration operation.
 
-  processing_spec_p->setRange(range_start, range_end);
+  processing_step_p->setSelectionPolygon(local_context.m_selectionPolygon);
+
+  // We must say what is the data source processing type.
+  processing_step_p->setSrcProcessingType(pappso::Axis::x, "MZ");
+
+  // It is essential that we document the relation between the axes and the
+  // corresponding data type.
+  processing_step_p->setDataKind(pappso::Axis::x, pappso::DataKind::mz);
+  processing_step_p->setDataKind(pappso::Axis::y, pappso::DataKind::unset);
 
   if(m_ui.integrateToRtPushButton->isChecked())
     {
       // qDebug() << "Integrating to XIC chromatogram.";
 
-      processing_step_p->newSpec(ProcessingType("MZ_TO_RT"), processing_spec_p);
+      processing_step_p->setDestProcessingType("RT");
 
       local_processing_flow.push_back(processing_step_p);
 
@@ -923,25 +951,22 @@ MassSpecTracePlotCompositeWidget::integrationRequested(
     }
   else if(m_ui.integrateToMzPushButton->isChecked() ||
           m_ui.integrateToDtMzPushButton->isChecked() ||
-          m_ui.integrateToRtMzPushButton->isChecked())
+          m_ui.integrateToMzRtPushButton->isChecked())
     {
       if(m_ui.integrateToMzPushButton->isChecked())
         {
           // qDebug() << "Integrating to mass spectrum.";
-          processing_step_p->newSpec(ProcessingType("MZ_TO_MZ"),
-                                     processing_spec_p);
+          processing_step_p->setDestProcessingType("MZ");
         }
-      else if(m_ui.integrateToRtMzPushButton->isChecked())
+      else if(m_ui.integrateToMzRtPushButton->isChecked())
         {
           // qDebug() << "Integrating to rt|m/z color map.";
-          processing_step_p->newSpec(ProcessingType("MZ_TO_RT_MZ"),
-                                     processing_spec_p);
+          processing_step_p->setDestProcessingType("MZ_RT");
         }
       else if(m_ui.integrateToDtMzPushButton->isChecked())
         {
           // qDebug() << "Integrating to dt|m/z color map.";
-          processing_step_p->newSpec(ProcessingType("MZ_TO_DT_MZ"),
-                                     processing_spec_p);
+          processing_step_p->setDestProcessingType("DT_MZ");
         }
 
       // At this point we need to define how the mass data integrator will
@@ -962,8 +987,11 @@ MassSpecTracePlotCompositeWidget::integrationRequested(
           // qDebug()
           //<< "The processing flow's default mz integ params are not valid.";
 
-          mz_integration_params =
-            *(local_processing_flow.mostRecentMzIntegrationParams());
+          const pappso::MzIntegrationParams *mz_integration_params_p =
+            local_processing_flow.mostRecentMzIntegrationParams();
+
+          if(mz_integration_params_p != nullptr)
+            mz_integration_params = *mz_integration_params_p;
         }
 
       // If it is not valid, finally resort to the parameters that can be
@@ -985,7 +1013,7 @@ MassSpecTracePlotCompositeWidget::integrationRequested(
       // Note that we cannot integrate mass spectra with a high resolution
       // here. We need to reduce the resolution to integer resolution.
       if(m_ui.integrateToDtMzPushButton->isChecked() ||
-         m_ui.integrateToRtMzPushButton->isChecked())
+         m_ui.integrateToMzRtPushButton->isChecked())
         {
           mz_integration_params.setBinningType(pappso::BinningType::NONE);
           mz_integration_params.setDecimalPlaces(0);
@@ -1016,49 +1044,40 @@ MassSpecTracePlotCompositeWidget::integrationRequested(
           static_cast<MassSpecTracePlotWnd *>(mp_parentWnd)
             ->integrateToDtMz(plottable_p, local_processing_flow);
         }
-      else if(m_ui.integrateToRtMzPushButton->isChecked())
+      else if(m_ui.integrateToMzRtPushButton->isChecked())
         {
           static_cast<MassSpecTracePlotWnd *>(mp_parentWnd)
-            ->integrateToRtMz(plottable_p, local_processing_flow);
+            ->integrateToMzRt(plottable_p, local_processing_flow);
         }
     }
   else if(m_ui.integrateToDtPushButton->isChecked())
     {
-      // qDebug() << "Integrating to drift spectrum.";
+       qDebug() << "Integrating to drift spectrum.";
 
-      processing_step_p->newSpec(ProcessingType("MZ_TO_DT"), processing_spec_p);
+      processing_step_p->setDestProcessingType("DT");
 
       local_processing_flow.push_back(processing_step_p);
 
       static_cast<MassSpecTracePlotWnd *>(mp_parentWnd)
         ->integrateToDt(plottable_p, local_processing_flow);
     }
-  else if(m_ui.integrateToRtDtPushButton->isChecked())
+  else if(m_ui.integrateToDtRtPushButton->isChecked())
     {
       // qDebug() << "Integrating to TIC|XIC chromatogram / drift spectrum
       // colormap.";
 
-
-      ProcessingSpec *spec_p = new ProcessingSpec();
-
-      processing_step_p->newSpec(ProcessingType("MZ_TO_RT_DT"), spec_p);
-
-      // We now need to specify the MZ range.
-
-      processing_step_p->newSpec(ProcessingType("MZ_TO_ANY"),
-                                 processing_spec_p);
+      processing_step_p->setDestProcessingType("DT_RT");
 
       local_processing_flow.push_back(processing_step_p);
 
       static_cast<MassSpecTracePlotWnd *>(mp_parentWnd)
-        ->integrateToRtDt(plottable_p, local_processing_flow);
+        ->integrateToDtRt(plottable_p, local_processing_flow);
     }
   else if(m_ui.integrateToIntPushButton->isChecked())
     {
       // qDebug() << "Integrating to TIC intensity.";
 
-      processing_step_p->newSpec(ProcessingType("MZ_TO_INT"),
-                                 processing_spec_p);
+      processing_step_p->setDestProcessingType("INT");
 
       local_processing_flow.push_back(processing_step_p);
 
@@ -1162,7 +1181,8 @@ MassSpecTracePlotCompositeWidget::addTrace(
               "integration "
               "params are not available in the processing flow.");
 
-          if(mz_integration_params_p->getBinningType() == pappso::BinningType::NONE)
+          if(mz_integration_params_p->getBinningType() ==
+             pappso::BinningType::NONE)
             {
               pappso::TraceMinusCombiner trace_minus_combiner;
               pappso::TracePlusCombiner trace_plus_combiner;
@@ -1595,14 +1615,17 @@ MassSpecTracePlotCompositeWidget::craftAnalysisStanza(
               // If the deconvolution was successful, than lastMz is the right
               // value. Otherwise it is the lastCursorHoveredPoint.
 
-              if(context.lastMz != std::numeric_limits<double>::min())
+              if(context.m_lastMz != std::numeric_limits<double>::min())
                 {
-                  stanza += QString("%1").arg(context.lastMz, 0, 'g', 6);
+                  stanza += QString("%1").arg(context.m_lastMz, 0, 'g', 6);
                 }
               else
                 {
                   stanza += QString("%1").arg(
-                    context.baseContext.lastCursorHoveredPoint.x(), 0, 'g', 6);
+                    context.m_baseContext.m_lastCursorHoveredPoint.x(),
+                    0,
+                    'g',
+                    6);
                 }
 
               prevChar = ' ';
@@ -1614,12 +1637,12 @@ MassSpecTracePlotCompositeWidget::craftAnalysisStanza(
               // not as an integration to INT, but simply as the y coordinate
               // of the point at X.
 
-              if(context.lastMz != std::numeric_limits<double>::min())
+              if(context.m_lastMz != std::numeric_limits<double>::min())
                 {
                   stanza += QString("%1").arg(
                     static_cast<pappso::MassSpecTracePlotWidget *>(
                       mp_plotWidget)
-                      ->getYatX(context.lastMz,
+                      ->getYatX(context.m_lastMz,
                                 static_cast<QCPGraph *>(plottable_p)),
                     0,
                     'g',
@@ -1630,8 +1653,9 @@ MassSpecTracePlotCompositeWidget::craftAnalysisStanza(
                   stanza += QString("%1").arg(
                     static_cast<pappso::MassSpecTracePlotWidget *>(
                       mp_plotWidget)
-                      ->getYatX(context.baseContext.lastCursorHoveredPoint.x(),
-                                static_cast<QCPGraph *>(plottable_p)),
+                      ->getYatX(
+                        context.m_baseContext.m_lastCursorHoveredPoint.x(),
+                        static_cast<QCPGraph *>(plottable_p)),
                     0,
                     'g',
                     6);
@@ -1643,7 +1667,7 @@ MassSpecTracePlotCompositeWidget::craftAnalysisStanza(
           if(curChar == 'x')
             {
               stanza +=
-                QString("%1").arg(context.baseContext.xDelta, 0, 'g', 6);
+                QString("%1").arg(context.m_baseContext.m_xDelta, 0, 'g', 6);
               prevChar = ' ';
               continue;
             }
@@ -1656,14 +1680,14 @@ MassSpecTracePlotCompositeWidget::craftAnalysisStanza(
             }
           if(curChar == 'z')
             {
-              stanza += QString("%1").arg(context.lastZ);
+              stanza += QString("%1").arg(context.m_lastZ);
 
               prevChar = ' ';
               continue;
             }
           if(curChar == 'M')
             {
-              stanza += QString("%1").arg(context.lastMr, 0, 'f', 6);
+              stanza += QString("%1").arg(context.m_lastMr, 0, 'f', 6);
 
               prevChar = ' ';
               continue;
@@ -1678,7 +1702,7 @@ MassSpecTracePlotCompositeWidget::craftAnalysisStanza(
           if(curChar == 'b')
             {
               stanza += QString("%1").arg(
-                context.baseContext.xRegionRangeStart, 0, 'f', 3);
+                context.m_baseContext.m_xRegionRangeStart, 0, 'f', 3);
 
               prevChar = ' ';
               continue;
@@ -1686,7 +1710,7 @@ MassSpecTracePlotCompositeWidget::craftAnalysisStanza(
           if(curChar == 'e')
             {
               stanza += QString("%1").arg(
-                context.baseContext.xRegionRangeEnd, 0, 'f', 3);
+                context.m_baseContext.m_xRegionRangeEnd, 0, 'f', 3);
 
               prevChar = ' ';
               continue;
