@@ -46,8 +46,8 @@
 
 
 /////////////////////// Local includes
-#include "ProcessingStep.hpp"
 #include "MsRunDataSet.hpp"
+#include "MsFragmentationSpec.hpp"
 
 
 namespace msxps
@@ -55,6 +55,8 @@ namespace msxps
 namespace minexpert
 {
 
+class ProcessingType;
+class ProcessingStep;
 
 class ProcessingFlow : public std::vector<ProcessingStep *>
 {
@@ -70,18 +72,22 @@ class ProcessingFlow : public std::vector<ProcessingStep *>
 
   std::vector<ProcessingStep *> steps();
 
-  bool innermostRange(ProcessingType type_mask,
-                      std::pair<ProcessingType, ProcessingSpec *> &pair) const;
-  bool innermostRange(ProcessingType type_mask, double &start, double &end) const;
+  bool innermostRange(
+    ProcessingType src_processing_type_mask,
+    std::pair<double, double> &out_range,
+    std::vector<ProcessingStep> *out_processing_steps_p = nullptr,
+    bool store_only_2d_steps = true,
+    bool store_only_skewed = true) const;
 
-  bool innermostRtRange(std::pair<ProcessingType, ProcessingSpec *> &pair) const;
-	bool innermostRtRange(double &start, double &end) const;
+  bool innermostRange(
+    const QString src_processing_type_mask,
+    std::pair<double, double> &out_range,
+    std::vector<ProcessingStep> *out_processing_steps_p = nullptr,
+    bool store_only_2d_steps = true,
+    bool store_only_skewed = true) const;
 
-  bool innermostDtRange(std::pair<ProcessingType, ProcessingSpec *> &pair) const;
-	bool innermostDtRange(double &start, double &end) const;
 
-  bool innermostMzRange(std::pair<ProcessingType, ProcessingSpec *> &pair) const;
-	bool innermostMzRange(double &start, double &end) const;
+  bool hasOnly1DSteps();
 
   size_t greatestMsLevel() const;
 
@@ -89,6 +95,14 @@ class ProcessingFlow : public std::vector<ProcessingStep *>
   const ProcessingStep *closestOlderStep(const QDateTime &date_time) const;
   const ProcessingStep *mostRecentStep() const;
 
+  std::vector<const ProcessingStep *>
+  stepsMatchingDestType(const ProcessingType &processing_type,
+                bool with_valid_fragmentation_spec = false) const;
+
+  std::vector<const ProcessingStep *>
+  stepsMatchingDestType(const QString &processing_type,
+                bool with_valid_fragmentation_spec = false) const;
+
   void setDefaultMzIntegrationParams(
     const pappso::MzIntegrationParams &mz_integration_params);
   const pappso::MzIntegrationParams &getDefaultMzIntegrationParams() const;
@@ -106,10 +120,10 @@ class ProcessingFlow : public std::vector<ProcessingStep *>
   protected:
   MsRunDataSetCstSPtr mcsp_msRunDataSet = nullptr;
 
-  // Note that each step has an optional pappso::MzIntegrationParams *. This one is the
-  // default one when not a single step has mz integration params. This one is
-  // set automatically in the first graph obtained when displaying the TIC
-  // chromatogram right after having loaded the ms run from file.
+  // Note that each step has an optional pappso::MzIntegrationParams *. This one
+  // is the default one when not a single step has mz integration params. This
+  // one is set automatically in the first graph obtained when displaying the
+  // TIC chromatogram right after having loaded the ms run from file.
   pappso::MzIntegrationParams m_defaultMzIntegrationParams;
 
   // Same for MsFragmentationSpec
