@@ -33,6 +33,8 @@
 
 /////////////////////// StdLib includes
 #include <vector>
+#include <cmath>
+
 
 /////////////////////// OPENMP include
 #include <omp.h>
@@ -40,6 +42,7 @@
 /////////////////////// Qt includes
 #include <QDebug>
 #include <QThread>
+#include <QFile>
 
 
 /////////////////////// pappsomspp includes
@@ -49,6 +52,7 @@
 /////////////////////// Local includes
 #include "QualifiedMassSpectrumVectorMassDataIntegrator.hpp"
 #include "QualifiedMassSpectrumVectorMassDataIntegratorToMz.hpp"
+#include "ProcessingStep.hpp"
 
 
 int qualifiedMassSpectrumVectorMassDataIntegratorToMzMetaTypeId =
@@ -105,20 +109,11 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::
 
 
 void
-QualifiedMassSpectrumVectorMassDataIntegratorToMz::integrateToMz()
+QualifiedMassSpectrumVectorMassDataIntegratorToMz::integrate()
 {
-  // qDebug();
-
   std::chrono::system_clock::time_point chrono_start_time =
     std::chrono::system_clock::now();
 
-  if(!mcsp_qualifiedMassSpectraToIntegrateVector->size())
-    {
-      qDebug() << "The qualified mass spectrum vector is empty, nothing to do.";
-
-      emit cancelOperationSignal();
-    }
-
   // Nothing special to do about processing flow info because that must have
   // been set before calling this function.
 
@@ -153,9 +148,23 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::integrateToMz()
 bool
 QualifiedMassSpectrumVectorMassDataIntegratorToMz::integrateToMzNoBinning()
 {
-  // qDebug();
+  qDebug();
 
-  emit setTaskDescriptionTextSignal("Integrating to a mass spectrum");
+  // If there are no data, nothing to do.
+  std::size_t mass_spectra_count =
+    mcsp_qualifiedMassSpectraToIntegrateVector->size();
+
+  if(!mass_spectra_count)
+    {
+      qDebug() << "The qualified mass spectrum vector is empty, nothing to do.";
+
+      emit cancelOperationSignal();
+    }
+
+  qDebug() << "The number of mass spectra to handle:" << mass_spectra_count;
+
+  emit setTaskDescriptionTextSignal(
+    "Integrating to a mass spectrum (no binning)");
 
   // We need a processing flow to work, and, in particular, a processing step
   // from which to find the specifics of the calculation.
@@ -163,24 +172,78 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::integrateToMzNoBinning()
   if(!m_processingFlow.size())
     qFatal("The processing flow cannot be empty. Program aborted.");
 
-  // qDebug().noquote() << "The processing flow has" << m_processingFlow.size()
-  //<< "steps and is:" << m_processingFlow.toString();
+  // When constructing this integrator, the processing flow passed as parameter
+  // was used to gather innermost ranges data. When the steps that match
+  // the innermost criterion, these are stored in m_innermostSteps2D. When the
+  // steps are non-2D steps, then only the m_xxRange values are updated.
 
-  // Get the most recent step that holds all the specifics of this integration.
-  const ProcessingStep *most_recent_processing_step_p =
-    m_processingFlow.mostRecentStep();
+  // If the m_innermostSteps2D vector of ProcessingSteps is non-empty, then,
+  // that means that we may have some MZ-related 2D processing steps to account
+  // at combination time, because that is precisely the only moment that we can
+  // do that work. To be able to provide selection polygon an mass
+  // spectrum-specific dt|rt data, we have the SelectionPolygonSpec class. There
+  // can be as many such class instances as required to document the integration
+  // boundaries rt/mz and dt/mz for example. This is why we need to fill in the
+  // vector of SelectionPolygonSpec and then set that to the combiner.
 
-  // There must be at least one ProcessingSpec (and thus a type) in the
-  // processing step.
+  // If the m_innermostSteps2D vector of ProcessingSteps is non-empty, then,
+  // that means that we may have some MZ-related 2D processing steps to account
+  // at combination time, because that is precisely the only moment that we can
+  // do that work.
 
-  if(!most_recent_processing_step_p->processingTypes().size())
-    qFatal("The processing step cannot be empty. Program aborted.");
+  qDebug() << "Now filling-in the selectionpolygonspecs.";
 
-  // We are integrating to m/z so we need a spec description that matches
-  // "ANY_TO_MZ".
+  std::vector<pappso::SelectionPolygonSpec> selection_polygon_specs;
 
-  if(!most_recent_processing_step_p->matches("ANY_TO_MZ"))
-    qFatal("There should be one ProcessingType = ANY_TO_MZ. Program aborted.");
+  std::size_t selection_polygon_spec_count =
+    fillInSelectionPolygonSpecs(selection_polygon_specs);
+
+  qDebug() << selection_polygon_spec_count
+           << "selection polygon specs were retained for the configuration of "
+              "the filter.";
+
+  // Prepare the template for all the combiners we'll use in the different
+  // threads.
+
+  pappso::MzIntegrationParams mz_integration_params =
+    m_processingFlow.getDefaultMzIntegrationParams();
+
+  pappso::TracePlusCombiner template_plus_combiner(
+    mz_integration_params.getDecimalPlaces());
+
+  // Now we need to check a number of things:
+  //
+  // 1. If there are no selection polygon specs, then all we need to check is if
+  // there is some m/z range limitation. If so, create the right filter.
+  //
+  // 2. If there is at least one selection polygon spec, then create the right
+  // filter.
+
+  // if(!selection_polygon_specs.size())
+  //{
+  // if(!std::isnan(m_mzRange.first) && !std::isnan(m_mzRange.second))
+  //{
+  // pappso::FilterInterfaceCstSPtr filter_csp =
+  // std::make_shared<const pappso::FilterResampleKeepXRange>(
+  // m_mzRange.first, m_mzRange.second);
+
+  // qDebug() << "Adding pappso::FilterResampleKeepXRange.";
+
+  // template_plus_combiner.addFilter(filter_csp);
+  //}
+  //}
+  // else
+  //{
+  // pappso::FilterInterfaceCstSPtr filter_csp =
+  // std::make_shared<const pappso::FilterResampleKeepPointInPolygon>(
+  // pappso::FilterResampleKeepPointInPolygon(selection_polygon_specs));
+
+  // qDebug() << "Adding pappso::FilterResampleKeepPointInPolygon.";
+
+  // template_plus_combiner.addFilter(filter_csp);
+  //}
+
+  qDebug();
 
   // At this point, allocate a visitor that is specific for the calculation of
   // the mass spectrum. Since we are not binning, we can use a visitor that
@@ -196,8 +259,7 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::integrateToMzNoBinning()
   // In the pair below, first is the ideal number of threads and second is the
   // number of mass spectra per thread.
   std::pair<std::size_t, std::size_t> best_parallel_params =
-    bestParallelIntegrationParams(
-      mcsp_qualifiedMassSpectraToIntegrateVector->size());
+    bestParallelIntegrationParams(mass_spectra_count);
 
   // Handy alias.
   using MassSpecVector = std::vector<pappso::QualifiedMassSpectrumCstSPtr>;
@@ -218,8 +280,9 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::integrateToMzNoBinning()
     vector_of_map_trace.push_back(std::make_shared<pappso::MapTrace>());
 
   // Now that we have all the parallel material, we can start the parallel work.
-  emit setProgressBarMaxValueSignal(
-    mcsp_qualifiedMassSpectraToIntegrateVector->size());
+  emit setProgressBarMaxValueSignal(mass_spectra_count);
+
+  qDebug() << "Starting the iteration in the visitors.";
 
   omp_set_num_threads(iterators.size());
 #pragma omp parallel for ordered
@@ -235,34 +298,24 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::integrateToMzNoBinning()
 
       pappso::MapTraceSPtr current_map_trace_sp = vector_of_map_trace.at(iter);
 
+      // std::size_t qualified_mass_spectrum_count =
+      // std::distance(current_iterator, end_iterator);
+
+      // qDebug() << "For thread index:" << iter
+      //<< "the number of mass spectra to process:"
+      //<< qualified_mass_spectrum_count;
+
       // Allocate the combiner because we'll need it for the checking of the
       // steps (will set eventual processing to the Trace if there are
       // start-end ranges for the mz values, for example with
       // setFilterResampleKeepXRange.
 
-      pappso::TracePlusCombiner trace_plus_combiner;
-
-      trace_plus_combiner.setDecimalPlaces(
-        m_processingFlow.getDefaultMzIntegrationParams().getDecimalPlaces());
-
-      // We need to inform the combiner if we have to limit the mz range
-      // for the combinations.
-
-      // These values are set in the in the base class constuctor.
-
-      if(m_mzRange.first != std::numeric_limits<double>::max() &&
-         m_mzRange.second != std::numeric_limits<double>::min())
-        {
-          //qDebug() << "The m_mzRange.first:" << m_mzRange.first
-                   //<< "and m_mzRange.second:" << m_mzRange.second;
-
-          trace_plus_combiner.setFilterResampleKeepXRange(
-            pappso::FilterResampleKeepXRange(m_mzRange.first,
-                                             m_mzRange.second));
-        }
-      //else
-        //qDebug() << "There is no limitation on the m/z range.";
+      pappso::TracePlusCombiner trace_plus_combiner(template_plus_combiner);
 
+      // Now process each qualified mass spectrum in the iterator range.
+      // Remember the begin and end iterators point to a range of mass
+      // spectra in the mass spectra vector that holds *all* the mass
+      // spectra to be processed.
       while(current_iterator != end_iterator)
         {
           // We want to be able to intercept any cancellation of the
@@ -282,6 +335,11 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::integrateToMzNoBinning()
           qualified_mass_spectrum_csp =
             checkQualifiedMassSpectrum(qualified_mass_spectrum_csp);
 
+          if(qualified_mass_spectrum_csp == nullptr)
+            qFatal(
+              "Programming error. Not possible that shared pointer to "
+              "qualified mass spectrum be nullptr.");
+
           // Great, at this point we actually have the mass spectrum!
 
           // We now need to ensure that the processing flow allows for this mass
@@ -291,74 +349,80 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::integrateToMzNoBinning()
           // that matches the greatest MS level found in the member processing
           // flow instance.
 
-          bool check_failed = false;
-
           if(!checkMsLevel(qualified_mass_spectrum_csp))
             {
-              // qDebug() << "The MsLevel check returned false";
-              check_failed = true;
+              qDebug() << "The MsLevel check returned false";
+              ++current_iterator;
+              continue;
             }
           if(!checkMsFragmentation(qualified_mass_spectrum_csp))
             {
-              // qDebug() << "The MsFragmentation check returned false";
-              check_failed = true;
+              qDebug() << "The MsFragmentation check returned false";
+              ++current_iterator;
+              continue;
             }
+
+          // Make easy check about RT and DT.
+
           if(!checkRtRange(qualified_mass_spectrum_csp))
             {
-              // qDebug() << "The RtRange check returned false";
-              check_failed = true;
+              qDebug() << "checkRtRange failed check.";
+
+              ++current_iterator;
+              continue;
             }
           if(!checkDtRange(qualified_mass_spectrum_csp))
             {
-              // qDebug() << "The DtRange check returned false";
-              check_failed = true;
-            }
-
-          if(check_failed)
-            {
-              // qDebug() << "The checks returned false";
+              qDebug() << "checkDtRange failed check.";
 
               ++current_iterator;
               continue;
             }
 
-#if 0 
+          pappso::Trace filtered_trace(
+            *qualified_mass_spectrum_csp->getMassSpectrumSPtr());
 
-          // This code is no more necessary because we handle all checks above
-          // and both are equipotent unless the code above is much quicker
-          // because it avoids cycling too much into the various steps/specs by
-          // looking at innermost Dt|Mz|Rt ranges. 
-          
-          // At this point we need to go deep in the processing flow's steps to
-          // check if the node matches the whole set of the steps' specs.
+          // It may be possible that the 2D steps limiting the m/z range
+          // have been followed by 1D steps further limiting the m/Z range.
+          // The point is, the 1D steps are not stored in
+          // selection_polygon_specs. However, by necessity, if there is at
+          // least one selection_polygon_spec, then the m_mzRange holds
+          // values. Futher, if there had been a 1D step limiting even more
+          // the m/z range, then that step would have been accounted for in
+          // m_mzRange. So we can account that limit right away.
 
-          // qDebug() << "The processing flow has" << m_processingFlow.size() <<
-          // "steps.";
-
-          // Boolean value needed because we need to break two loops.
-          bool should_increment_iterator_and_continue = false;
-
-          for(auto &&step : m_processingFlow)
+          if(!std::isnan(m_mzRange.first) && !std::isnan(m_mzRange.second))
             {
-              if(!checkProcessingStep(*step,
-                                      qualified_mass_spectrum_csp,
-                                      mass_spectrum_plus_combiner))
-                {
-                  should_increment_iterator_and_continue = true;
-                  break;
-                }
+              pappso::FilterResampleKeepXRange the_keep_filter(
+                m_mzRange.first, m_mzRange.second);
+
+              filtered_trace = the_keep_filter.filter(filtered_trace);
             }
 
-          if(should_increment_iterator_and_continue)
+          if(selection_polygon_specs.size())
             {
-              ++current_iterator;
-              continue;
-            }
-#endif
+              pappso::FilterResampleKeepPointInPolygon the_polygon_filter(
+                selection_polygon_specs);
 
-          trace_plus_combiner.combine(
-            *current_map_trace_sp,
-            *qualified_mass_spectrum_csp->getMassSpectrumCstSPtr());
+              filtered_trace = the_polygon_filter.filter(
+                filtered_trace,
+                qualified_mass_spectrum_csp->getDtInMilliSeconds(),
+                qualified_mass_spectrum_csp->getRtInMinutes());
+            }
+
+          // No need to configure the combiner here, because we already have
+          // done that above.
+
+          // qDebug() << "Going to combine a new qualified mass spectrum.";
+
+          std::size_t before_size = current_map_trace_sp->size();
+
+          trace_plus_combiner.combine(*current_map_trace_sp, filtered_trace);
+
+          std::size_t after_size = current_map_trace_sp->size();
+
+          qDebug() << "Before combination, map trace map size: " << before_size
+                   << "after combination:" << after_size;
 
           // qDebug() << "Processed one more spectrum.";
 
@@ -442,25 +506,41 @@ bool
 QualifiedMassSpectrumVectorMassDataIntegratorToMz::
   integrateToMzArbitraryBinning()
 {
-  // qDebug();
+  qDebug();
+
+  // If there are no data, nothing to do.
+  std::size_t mass_spectra_count =
+    mcsp_qualifiedMassSpectraToIntegrateVector->size();
+
+  if(!mass_spectra_count)
+    {
+      qDebug() << "The qualified mass spectrum vector is empty, nothing to do.";
+
+      emit cancelOperationSignal();
+    }
+
+  qDebug() << "The number of mass spectra to handle:" << mass_spectra_count;
+
+  emit setTaskDescriptionTextSignal("Integrating to a mass spectrum (binning)");
 
   //  This function needs to be called by integrateToMz() that has setup all the
   //  required parameters.
 
-  emit setTaskDescriptionTextSignal("Integrating to a mass spectrum");
-
   // We need a processing flow to work, and, in particular, a processing step
   // from which to find the specifics of the calculation.
 
   if(!m_processingFlow.size())
     qFatal("The processing flow cannot be empty. Program aborted.");
 
+  // In this case where binning is requested, we need to craft the bins, and for
+  // that we need proper mz integration params.
+
   pappso::MzIntegrationParams local_mz_integration_params =
     m_processingFlow.getDefaultMzIntegrationParams();
 
   if(!local_mz_integration_params.isValid())
     {
-      qDebug() << "The mz integration paramas are not valid.";
+      qDebug() << "The mz integration params are not valid.";
 
       // If the mz integration params are not valid, then we cannot know the
       // start and end of the bins ! We need to resort to the statistical data
@@ -479,28 +559,98 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::
   // Now we can create the bins!
   std::vector<double> bins = local_mz_integration_params.createBins();
 
-  // qDebug() << "Created bins at:"
-  //<< QDateTime::currentDateTime().toString("yyyyMMdd-HH-mm-ss");
+  QString bins_before = "Bins:\n";
+  for(auto &&bin : bins)
+    bins_before += QString("%1\n").arg(bin, 0, 'f', 6);
 
-  // At this point, allocate a visitor that is specific for the calculation of
-  // the mass spectrum. Since we are not binning, we can use a visitor that
-  // integrates to a pappso::Trace, not a pappso::MassSpectrum.
+    // qDebug() << "Created bins at:"
+    //<< QDateTime::currentDateTime().toString("yyyyMMdd-HH-mm-ss");
 
-  // But we want to parallelize the computation. Se we will allocate a vector of
-  // visitors (as many as possible for the available processor threads), each
-  // visitor having a subset of the initial data to integrate.
 
-  emit setProgressBarMaxValueSignal(
-    mcsp_qualifiedMassSpectraToIntegrateVector->size());
+#if 0
+  QString fileName = "/tmp/massSpecArbitraryBins.txt-at-" +
+                     QDateTime::currentDateTime().toString("yyyyMMdd-HH-mm-ss");
 
-  // Determine the best number of threads to use and how many spectra to provide
-  // each of the threads.
+  qDebug() << "Writing the list of bins setup in the "
+              "mass spectrum in file "
+           << fileName;
 
-  // In the pair below, first is the ideal number of threads and second is the
-  // number of mass spectra per thread.
+  QFile file(fileName);
+  file.open(QIODevice::WriteOnly);
+
+  QTextStream fileStream(&file);
+
+  for(auto &&bin : bins)
+    fileStream << QString("%1\n").arg(bin, 0, 'f', 10);
+
+  fileStream.flush();
+  file.close();
+#endif
+
+
+  // When constructing this integrator, the processing flow passed as parameter
+  // was used to gather innermost ranges data. When the steps that match
+  // the innermost criterion, these are stored in m_innermostSteps2D. When the
+  // steps are non-2D steps, then only the m_xxRange values are updated.
+
+  // If the m_innermostSteps2D vector of ProcessingSteps is non-empty, then,
+  // that means that we may have some MZ-related 2D processing steps to account
+  // at combination time, because that is precisely the only moment that we can
+  // do that work. To be able to provide selection polygon an mass
+  // spectrum-specific dt|rt data, we have the SelectionPolygonSpec class. There
+  // can be as many such class instances as required to document the integration
+  // boundaries rt/mz and dt/mz for example. This is why we need to fill in the
+  // vector of SelectionPolygonSpec and then set that to the combiner.
+
+  // If the m_innermostSteps2D vector of ProcessingSteps is non-empty, then,
+  // that means that we may have some MZ-related 2D processing steps to account
+  // at combination time, because that is precisely the only moment that we can
+  // do that work.
+
+  qDebug() << "Now filling-in the selectionpolygonspecs.";
+
+  std::vector<pappso::SelectionPolygonSpec> selection_polygon_specs;
+
+  std::size_t selection_polygon_spec_count =
+    fillInSelectionPolygonSpecs(selection_polygon_specs);
+
+  qDebug() << selection_polygon_spec_count
+           << "selection polygon specs were retained for the configuration of "
+              "the filter.";
+
+  // Prepare the template for all the combiners we'll use in the different
+  // threads.
+
+  pappso::MassSpectrumPlusCombiner template_plus_combiner(
+    local_mz_integration_params.getDecimalPlaces());
+
+  template_plus_combiner.setBins(bins);
+
+  QString bins_after = "Bins:\n";
+  for(auto &&bin : template_plus_combiner.getBins())
+    bins_after += QString("%1\n").arg(bin, 0, 'f', 6);
+
+  if(bins_before == bins_after)
+    qDebug() << "The copying of the bins in the template combiner worked.";
+  else
+    qFatal("Programming error.");
+
+  // At this point, allocate a visitor that is specific for the calculation
+  // of the mass spectrum. Since we are not binning, we can use a visitor
+  // that integrates to a pappso::Trace, not a pappso::MassSpectrum.
+
+  // But we want to parallelize the computation. Se we will allocate a
+  // vector of visitors (as many as possible for the available processor
+  // threads), each visitor having a subset of the initial data to
+  // integrate.
+
+  // Determine the best number of threads to use and how many spectra to
+  // provide each of the threads.
+
+  // In the pair below, first is the ideal number of threads and second is
+  // the number of mass spectra per thread.
   std::pair<std::size_t, std::size_t> best_parallel_params =
-    bestParallelIntegrationParams(
-      mcsp_qualifiedMassSpectraToIntegrateVector->size());
+    bestParallelIntegrationParams(mass_spectra_count);
 
   using MassSpecVector = std::vector<pappso::QualifiedMassSpectrumCstSPtr>;
   using VectorIterator = MassSpecVector::const_iterator;
@@ -511,18 +661,20 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::
     calculateIteratorPairs(best_parallel_params.first,
                            best_parallel_params.second);
 
-  // Set aside a vector pappso::MapTrace shared pointers that will be getting
-  // the result of the integrations. There will be one such instance per thread
-  // executing.
+  // Set aside a vector pappso::MapTrace shared pointers that will be
+  // getting the result of the integrations. There will be one such instance
+  // per thread executing.
 
   std::vector<pappso::MapTraceSPtr> vector_of_map_trace;
   for(std::size_t iter = 0; iter < iterators.size(); ++iter)
     vector_of_map_trace.push_back(std::make_shared<pappso::MapTrace>());
 
+  // Now that we have all the parallel material, we can start the parallel work.
+  emit setProgressBarMaxValueSignal(mass_spectra_count);
+
+  qDebug() << "Starting the iteration in the visitors.";
+
   omp_set_num_threads(iterators.size());
-
-  // qDebug() << "Will use" << iterators.size() << "threads";
-
 #pragma omp parallel for ordered
   for(std::size_t iter = 0; iter < iterators.size(); ++iter)
     {
@@ -530,9 +682,7 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::
 
       // Get the iterator range that we need to deal in this thread.
       VectorIterator current_iterator = iterators.at(iter).first;
-      // For debugging purposes
-      // VectorIterator end_iterator     = std::prev(iterators.at(iter).second);
-      VectorIterator end_iterator = iterators.at(iter).second;
+      VectorIterator end_iterator     = iterators.at(iter).second;
 
       // Get the map for this thread.
 
@@ -551,35 +701,12 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::
       // setFilterResampleKeepXRange.
 
       pappso::MassSpectrumPlusCombiner mass_spectrum_plus_combiner(
-        local_mz_integration_params.getDecimalPlaces());
-
-      // We need to inform the combiner if we have to limit the mz range
-      // for the combinations.
-
-      // These values are set in the in the base class constuctor.
-
-      if(m_mzRange.first != std::numeric_limits<double>::max() &&
-         m_mzRange.second != std::numeric_limits<double>::min())
-        {
-
-          // qDebug() << "The m_mzRange.first:" << m_mzRange.first
-          //<< "and m_mzRange.second:" << m_mzRange.second;
-
-          mass_spectrum_plus_combiner.setFilterResampleKeepXRange(
-            pappso::FilterResampleKeepXRange(m_mzRange.first,
-                                             m_mzRange.second));
-        }
-      // else
-      // qDebug() << "There is no limitation on the m/z range.";
-
-      // Configure the binning mass spectrum combiner.
-
-      mass_spectrum_plus_combiner.setBins(bins);
+        template_plus_combiner);
 
       // Now process each qualified mass spectrum in the iterator range.
-      // Remember the begin and end iterators point to a range of mass spectra
-      // in the mass spectra vector that holds *all* the mass spectra to be
-      // processed.
+      // Remember the begin and end iterators point to a range of mass
+      // spectra in the mass spectra vector that holds *all* the mass
+      // spectra to be processed.
       while(current_iterator != end_iterator)
         {
           // We want to be able to intercept any cancellation of the
@@ -593,101 +720,107 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::
           pappso::QualifiedMassSpectrumCstSPtr qualified_mass_spectrum_csp =
             *current_iterator;
 
-          if(qualified_mass_spectrum_csp == nullptr)
-            {
-              qFatal("The qualified_mass_spectrum_csp cannot be nullptr");
-            }
-          // else
-          // qDebug() << "Current mass spectrum is valid with rt:"
-          //<< qualified_mass_spectrum_csp->getRtInMinutes();
-
           // Ensure that the qualified mass spectrum contains a mass spectrum
-          // that effectively contains the binary (mz,i) data. In particular, if
-          // that mass spectrum is not there, that might be because the data are
-          // not loaded in memory but need to be read from the file.
+          // that effectively contains the binary (mz,i) data.
 
           qualified_mass_spectrum_csp =
             checkQualifiedMassSpectrum(qualified_mass_spectrum_csp);
 
+          if(qualified_mass_spectrum_csp == nullptr)
+            qFatal(
+              "Programming error. Not possible that shared pointer to "
+              "qualified mass spectrum be nullptr.");
+
           // Great, at this point we actually have the mass spectrum!
 
-          // We now need to ensure that the processing flow allows for this mass
-          // spectrum to be combined.
+          // We now need to ensure that the processing flow allows for this
+          // mass spectrum to be combined.
 
-          // Immediately check if the qualified mass spectrum is of a MS level
-          // that matches the greatest MS level found in the member processing
-          // flow instance.
-
-          bool check_failed = false;
+          // Immediately check if the qualified mass spectrum is of a MS
+          // level that matches the greatest MS level found in the member
+          // processing flow instance.
 
           if(!checkMsLevel(qualified_mass_spectrum_csp))
             {
-              // qDebug() << "The MsLevel check returned false";
-              check_failed = true;
+              //qDebug() << "The MsLevel check returned false";
+              ++current_iterator;
+              continue;
             }
           if(!checkMsFragmentation(qualified_mass_spectrum_csp))
             {
-              // qDebug() << "The MsFragmentation check returned false";
-              check_failed = true;
+              //qDebug() << "The MsFragmentation check returned false";
+              ++current_iterator;
+              continue;
             }
+
+          // Make easy check about RT and DT.
+
           if(!checkRtRange(qualified_mass_spectrum_csp))
             {
-              // qDebug() << "The RtRange check returned false";
-              check_failed = true;
+              //qDebug() << "checkRtRange failed check.";
+              ++current_iterator;
+              continue;
             }
           if(!checkDtRange(qualified_mass_spectrum_csp))
             {
-              // qDebug() << "The DtRange check returned false";
-              check_failed = true;
-            }
-
-          if(check_failed)
-            {
-              // qDebug() << "The checks returned false";
-
+              //qDebug() << "checkDtRange failed check.";
               ++current_iterator;
               continue;
             }
 
-#if 0 
+          pappso::Trace filtered_trace;
 
-          // This code is no more necessary because we handle all checks above
-          // and both are equipotent unless the code above is much quicker
-          // because it avoids cycling too much into the various steps/specs by
-          // looking at innermost Dt|Mz|Rt ranges. 
-          
-          // At this point we need to go deep in the processing flow's steps to
-          // check if the node matches the whole set of the steps' specs.
+          // It may be possible that the 2D steps limiting the m/z range
+          // have been followed by 1D steps further limiting the m/Z range.
+          // The point is, the 1D steps are not stored in
+          // selection_polygon_specs. However, by necessity, if there is at
+          // least one selection_polygon_spec, then the m_mzRange holds
+          // values. Futher, if there had been a 1D step limiting even more
+          // the m/z range, then that step would have been accounted for in
+          // m_mzRange. So we can account that limit right away.
 
-          // qDebug() << "The processing flow has" << m_processingFlow.size() <<
-          // "steps.";
-
-          // Boolean value needed because we need to break two loops.
-          bool should_increment_iterator_and_continue = false;
-
-          for(auto &&step : m_processingFlow)
+          if(!std::isnan(m_mzRange.first) && !std::isnan(m_mzRange.second))
             {
-              if(!checkProcessingStep(*step,
-                                      qualified_mass_spectrum_csp,
-                                      mass_spectrum_plus_combiner))
+              filtered_trace.initialize(
+                *qualified_mass_spectrum_csp->getMassSpectrumSPtr());
+
+              pappso::FilterResampleKeepXRange the_keep_filter(
+                m_mzRange.first, m_mzRange.second);
+
+              filtered_trace = the_keep_filter.filter(filtered_trace);
+            }
+
+          if(selection_polygon_specs.size())
+            {
+              pappso::FilterResampleKeepPointInPolygon the_polygon_filter(
+                selection_polygon_specs);
+
+              if(!filtered_trace.size())
                 {
-                  should_increment_iterator_and_continue = true;
-                  break;
+                  filtered_trace.initialize(
+                    *qualified_mass_spectrum_csp->getMassSpectrumSPtr());
                 }
+
+              filtered_trace = the_polygon_filter.filter(
+                filtered_trace,
+                qualified_mass_spectrum_csp->getDtInMilliSeconds(),
+                qualified_mass_spectrum_csp->getRtInMinutes());
             }
 
-          if(should_increment_iterator_and_continue)
-            {
-              ++current_iterator;
-              continue;
-            }
-#endif
+          // No need to configure the combiner here, because we already have
+          // done that above.
 
-          mass_spectrum_plus_combiner.combine(
-            *current_map_trace_sp,
-            *qualified_mass_spectrum_csp->getMassSpectrumCstSPtr());
+          // qDebug() << "Going to combine a new qualified mass spectrum.";
 
-          // qDebug() << "Processed one more spectrum:"
+          if(filtered_trace.size())
+            mass_spectrum_plus_combiner.combine(*current_map_trace_sp,
+                                                filtered_trace);
+          else
+            mass_spectrum_plus_combiner.combine(
+              *current_map_trace_sp,
+              *qualified_mass_spectrum_csp->getMassSpectrumSPtr());
+
+          // qDebug() << "Processed one more qualified mass spectrum:"
           //<< qualified_mass_spectrum_csp->getMassSpectrumSPtr().get()
           //<< "with size:" << qualified_mass_spectrum_csp->size();
 
@@ -719,11 +852,12 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::
   emit unlockTaskMonitorCompositeWidgetSignal();
   emit setupProgressBarSignal(0, vector_of_map_trace.size() - 1);
 
-  // We might be here because the user cancelled the operation in the for loop
-  // above (visiting all the visitors). In this case m_isOperationCancelled is
-  // true. We want to set it back to false, so that the following loop is gone
-  // through. The user can ask that the operation be cancelled once more. But
-  // we want that at least the performed work be used to show the trace.
+  // We might be here because the user cancelled the operation in the for
+  // loop above (visiting all the visitors). In this case
+  // m_isOperationCancelled is true. We want to set it back to false, so
+  // that the following loop is gone through. The user can ask that the
+  // operation be cancelled once more. But we want that at least the
+  // performed work be used to show the trace.
 
   pappso::MassSpectrumPlusCombiner mass_spectrum_plus_combiner(
     local_mz_integration_params.getDecimalPlaces());
@@ -733,7 +867,8 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::
   m_isOperationCancelled = false;
   int iter               = 0;
 
-  // qDebug() << "The vector of map traces has:" << vector_of_map_trace.size()
+  // qDebug() << "The vector of map traces has:" <<
+  // vector_of_map_trace.size()
   //<< "items.";
 
   for(auto &&current_map_trace_sp : vector_of_map_trace)
@@ -748,17 +883,17 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::
 
       emit setStatusTextSignal(message);
 
-      //qDebug() << message;
+      // qDebug() << message;
 
       mass_spectrum_plus_combiner.combine(m_mapTrace, *current_map_trace_sp);
 
       emit setProgressBarCurrentValueSignal(iter);
 
-      //qDebug() << "Emitted current value:" << iter;
+      // qDebug() << "Emitted current value:" << iter;
     }
 
   // Write the data to file so that we can check. It works !!!
-  // pappso::Utils::appendToFile(m_mapTrace.toString(), "/tmp/prova.txt");
+  pappso::Utils::appendToFile(m_mapTrace.toString(), "/tmp/prova.txt");
 
   // At this point, we have really combined all the traces into a single map
   // trace!
